package it.amattioli.applicate.sessions;

import java.io.Serializable;

import org.apache.commons.lang.StringUtils;

import it.amattioli.applicate.browsing.ObjectBrowser;
import it.amattioli.applicate.browsing.ObjectBrowserImpl;
import it.amattioli.dominate.Entity;
import it.amattioli.dominate.resolver.*;

/**
 * A service factory that create {@link ObjectBrowser} objects.<p>
 * 
 * The name of the service to be created must follow the pattern:
 * <ul>
 *   <li><i>entityName</i>ObjectBrowser
 * </ul>
 * where <i>entityName</i> is the name of the entity that has to be browsed
 * by the created browser.<p>
 * 
 * @author andrea
 *
 */
public class ObjectBrowserFactory implements ServiceFactory {
	private static final String SERVICE_NAME_SUFFIX = "ObjectBrowser";
	private EntityResolver entityResolver;

	public ObjectBrowserFactory() {
		CompositeEntityResolver entityResolver = new CompositeEntityResolver();
		entityResolver.loadChildren();
		this.entityResolver = entityResolver;
	}
	
	public ObjectBrowserFactory(EntityResolver entityResolver) {
		this.entityResolver = entityResolver;
	}
	
	@Override
	public Object createService(String serviceName) {
		Class entityClass = entityResolver.find(getEntityName(serviceName));
		return createService(entityClass);
	}

	public <I extends Serializable, T extends Entity<I>> ObjectBrowser<I,T> createService(Class<T> entityClass) {
		return new ObjectBrowserImpl<I,T>(entityClass);
	}
	
	private String getEntityName(String serviceName) {
		return StringUtils.substringBefore(serviceName, SERVICE_NAME_SUFFIX);
	}

	@Override
	public boolean knowsService(String serviceName) {
		return serviceName.endsWith(SERVICE_NAME_SUFFIX) &&
		       entityResolver.find(getEntityName(serviceName)) != null;
	}

}
